@@ -51,6 +51,7 @@ sub init {
   $this->{POSTCALL}  = $args{postcall};
   $this->{CLASS}     = $args{class};
   $this->{CATCH}     = $args{catch};
+  $this->{CONDITION} = $args{condition};
 
   if (ref($this->{CATCH})
       and @{$this->{CATCH}} > 1
@@ -168,9 +169,7 @@ C<length> feature.
 
 sub argument_style {
   my $this = shift;
-  foreach my $arg (@{$this->{ARGUMENTS}}) {
-    return 'ansi' if $arg->name =~ /length.*\(/;
-  }
+  return 'ansi' if $this->has_argument_with_length;
   return 'kr';
 }
 
@@ -284,25 +283,31 @@ sub print {
 
     $code .= "  $code_type:\n";
     $code .= "    try {\n";
-    $code .= '      ' . $precall if $precall;
+    if ($precall) {
+      $code .= '      ' . $precall;
+    }
     $code .= '      ' . $ccode . ";\n";
     if( $has_ret && defined $ret_typemap->output_code( '', '' ) ) {
-      $code .= '      ' . $ret_typemap->output_code( 'ST(0)', 'RETVAL' ) . ";\n";
+      my $retcode = $ret_typemap->output_code( 'ST(0)', 'RETVAL' );
+      $code .= '      ' . $retcode . ";\n";
     }
     if( $has_ret && defined $ret_typemap->output_list( '' ) ) {
-      $code .= '      ' . $ret_typemap->output_list( 'RETVAL' ) . ";\n";
+      my $retcode = $ret_typemap->output_list( 'RETVAL' );
+      $code .= '      ' . $retcode . ";\n";
     }
     $code .= "    }\n";
     my @catchers = @{$this->{EXCEPTIONS}};
     foreach my $exception_handler (@catchers) {
-      $code .= $exception_handler->handler_code;
+      my $handler_code = $exception_handler->handler_code;
+      $code .= $handler_code;
     }
 
     $output = "  OUTPUT: RETVAL\n" if $has_ret;
 
     if( $has_ret && defined $ret_typemap->cleanup_code( '', '' ) ) {
       $cleanup .= "  CLEANUP:\n";
-      $cleanup .= '    ' . $ret_typemap->cleanup_code( 'ST(0)', 'RETVAL' ) . ";\n";
+      my $cleanupcode = $ret_typemap->cleanup_code( 'ST(0)', 'RETVAL' );
+      $cleanup .= '    ' . $cleanupcode . ";\n";
     }
   }
 
@@ -335,14 +340,42 @@ $cur_module PACKAGE=$pcname
 EOT
   }
 
-  $out .= "$retstr\n";
-  $out .= "$fname($arg_list)\n";
-  $out .= $init;
-  $out .= $code;
-  $out .= $postcall;
-  $out .= $output;
-  $out .= $cleanup;
-  $out .= "\n";
+  my $head = "$retstr\n"
+             . "$fname($arg_list)\n";
+  my $body = $init . $code . $postcall . $output . $cleanup . "\n";
+  $this->_munge_code(\$body) if $this->has_argument_with_length;
+
+  $out .= $head . $body;
+  return $out;
+}
+
+# This replaces the use of "length(varname)" with
+# the proper name of the XS variable that is auto-generated in
+# case of the XS length() feature. The Argument's take care of
+# this and do nothing if they're not of the "length" type.
+# Any additional checking "$this->_munge_code(\$code) if $using_length"
+# is just an optimization!
+sub _munge_code {
+  my $this = shift;
+  my $code = shift;
+  
+  foreach my $arg (@{$this->{ARGUMENTS}}) {
+    $$code = $arg->fix_name_in_code($$code);
+  }
+}
+
+=head2 print_declaration
+
+Returns a string with a C++ method declaration for the node.
+
+=cut
+
+sub print_declaration {
+    my( $this ) = @_;
+
+    return $this->ret_type->print . ' ' . $this->cpp_name . '( ' .
+           join( ', ', map $_->print, @{$this->arguments} ) . ')' .
+           ( $this->const ? ' const' : '' );
 }
 
 =head2 perl_function_name
@@ -363,6 +396,22 @@ but overridden in the L<ExtUtils::XSpp::Node::Method> sub-class.
 
 sub is_method { 0 }
 
+=head2 has_argument_with_length
+
+Returns true if the function has any argument that uses the XS length
+feature.
+
+=cut
+
+sub has_argument_with_length {
+  my $this = shift;
+  foreach my $arg (@{$this->{ARGUMENTS}}) {
+    return 1 if $arg->uses_length;
+  }
+  return();
+}
+
+
 =begin documentation
 
 ExtUtils::XSpp::Node::_call_code( argument_string )
@@ -375,7 +424,6 @@ Return something like "foo( $argument_string )".
 
 sub _call_code { return $_[0]->cpp_name . '(' . $_[1] . ')'; }
 
-
 =head1 ACCESSORS
 
 =head2 cpp_name
@@ -403,6 +451,11 @@ Returns the C++ return type.
 
 Returns the C<%code> decorator if any.
 
+=head2 set_code
+
+Sets the implementation for the method call (equivalent to using
+C<%code>); takes the code as an array reference containing the lines.
+
 =head2 cleanup
 
 Returns the C<%cleanup> decorator if any.
@@ -419,11 +472,13 @@ with the function via C<%catch>. (array reference)
 =cut
 
 sub cpp_name { $_[0]->{CPP_NAME} }
+sub set_cpp_name { $_[0]->{CPP_NAME} = $_[1] }
 sub perl_name { $_[0]->{PERL_NAME} }
 sub set_perl_name { $_[0]->{PERL_NAME} = $_[1] }
 sub arguments { $_[0]->{ARGUMENTS} }
 sub ret_type { $_[0]->{RET_TYPE} }
 sub code { $_[0]->{CODE} }
+sub set_code { $_[0]->{CODE} = $_[1] }
 sub cleanup { $_[0]->{CLEANUP} }
 sub postcall { $_[0]->{POSTCALL} }
 sub catch { $_[0]->{CATCH} ? $_[0]->{CATCH} : [] }
@@ -431,8 +486,8 @@ sub catch { $_[0]->{CATCH} ? $_[0]->{CATCH} : [] }
 =head2 set_static
 
 Sets the C<static>-ness attribute of the function.
-Can be either undef (i.e. not static), C<package_static>,
-or C<class_static>.
+Can be either undef (i.e. not static), C<"package_static">,
+or C<"class_static">.
 
 =head2 package_static
 
